#include "api/df_access_control_lists.h"
#include "datafox_library_iv.h"

#if defined(_MSC_VER) || defined(__MINGW32__)
#define snprintf	    _snprintf
#endif

DF_API_EXTERN_DECORATE void DF_API_CALL DFCClearEntrance2ListBuffer( int connectionId, int number )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, number );

    // zulaessige Bereiche pruefen
    if ( number < -1 || number > 7 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "" );
        return;
    }

    channelBridge->ClearEntrance2ListBuffer( number );

    channelBridge->logEnd( "" );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeEntrance2List( int connectionId, int number, int recordCount, int size, const unsigned char *buffer, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i", __func__, connectionId, number, recordCount, size );

    // zulaessige Bereiche pruefen
    if ( number < 0 || number > 7 || buffer == nullptr || errorNumber == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( 0 );
        }

        channelBridge->logEnd( "\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // ListNum ist:
    // 0: Reader         (33 Byte)
    // 1: Identification (64 Byte)
    // 2: Location       (20 Byte)
    // 3: Time           (25 Byte)
    // 4: Holiday        (21 Byte)
    // 5: Events         (17 Byte)
    // 6: Action         (21 Byte)
    // 7: Action2        (39 Byte)

    // Bereich der gueltigen Nummer pruefen.
    switch ( number )
    {
    case 0:
        // Anzahl moeglicher Datensaetze
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_0))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Berechnete Listengroesze pruefen
        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_0) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 1:
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_1))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_1) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 2:
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_2))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_2) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 3:
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_3))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_3) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 4:
        // Anzahl moeglicher Datensaetze
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_4))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Berechnete Listengroesze pruefen
        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_4) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 5:
        // Anzahl moeglicher Datensaetze
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_5))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Berechnete Listengroesze pruefen
        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_5) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 6:
        // Anzahl moeglicher Datensaetze
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_6))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Berechnete Listengroesze pruefen
        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_6) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    case 7:
        // Anzahl moeglicher Datensaetze
        if ( (recordCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_7))*/ )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Berechnete Listengroesze pruefen
        if ( size != (recordCount * ENTRANCE2_RECORD_SIZE_7) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        break;

    default:
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
        break;
    }

    int iRes = 0;
    if ( channelBridge->getLastErrorNumber( 0 ) != IV_ERROR_SUCCESS )
    {
        *errorNumber = channelBridge->getLastErrorNumber( 0 );
    }
    else
    {
        iRes = channelBridge->MakeEntrance2List( number, recordCount, size, buffer );
    }

    channelBridge->logEnd( "\t\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadEntrance2List( int connectionId, int deviceAddress, int number, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || number < -1 || number > 7 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Anderer Blocktransfer besteht bereits?
    if ( channelBridge->getBlockTransferState( deviceAddress ) == controller_segment::bts_active 
        && channelBridge->getBlockTransferType( deviceAddress ) != controller_segment::btt_access_control_list )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_BLOCK_TRANSFER_ACTIVE );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Zutrittskontrolle Version 2 wird erst ab 04.01.xx unterstuetzt.
    unsigned char protocolVersion = channelBridge->getProtocolVersion( deviceAddress );
    if ( protocolVersion < 3 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_VERSION_FIRMWARE );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes;
    /*
    char version[32];
    memset(version, 0, sizeof(version));
    // Firmwareversion lesen
    if ((iRes = channelBridge->getVersionFirmware(deviceAddress, version)) != IV_ERROR_SUCCESS)
    {
        channelBridge->setLastErrorNumber(deviceAddress, (iRes < 0) ? -iRes : IV_ERROR_VERSION_FIRMWARE);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }
    
    // Firmwareversion pruefen
    if (!theApp.m_util.CheckVersionFirmwareZK(version)) {

    *piError = IV_ERROR_FUNCTION_UNAVAILABLE;
    channelBridge->logEnd("\t%i\t(%i)", *piError, 0);
    return 0;
    }
    */
    // Listendaten wegschreiben. BIN
    if ( channelBridge->logIsWriteList() && channelBridge->getBlockTransferState( deviceAddress ) == controller_segment::bts_inactive )
    {
        char fileNameBin[_MAX_PATH];
        char basePath[340];
        int listNum;
        time_t t;
        tm *ltm;

        t = time( nullptr );
        ltm = localtime( &t );

        for ( listNum = 0; listNum < 8; listNum++ )
        {
            // Dateiname der Logdatei
            snprintf( fileNameBin, sizeof( fileNameBin ) - 1, LOG_LIST_FILENAME_BIN, channelBridge->logGetBasePath( basePath ),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, 'E', listNum );

            // Daten in Datei schreiben.
            channelBridge->WriteListEntrance2ToBinFile( listNum, fileNameBin );
        }
    }

    // Listendaten wegschreiben. TXT
    if ( channelBridge->logIsWriteList() > 1 && channelBridge->getBlockTransferState( deviceAddress ) == controller_segment::bts_inactive )
    {
        char fileNameTxt[_MAX_PATH];
        char basePath[340];
        std::string listName;
		unsigned char fieldCount, varList;
        int listNum;
        time_t t;
        tm *ltm;

        t = time( nullptr );
        ltm = localtime( &t );

        for ( listNum = 0; listNum < 8; listNum++ )
        {
            if ( !channelBridge->GetTableRecord( 200 + listNum, listName, &fieldCount, &varList, TRUE ) )
                break;

            // Dateiname der Logdatei
            snprintf( fileNameTxt, sizeof( fileNameTxt ) - 1, LOG_LIST_FILENAME_TXT, channelBridge->logGetBasePath( basePath ),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, 'E', listName.c_str() );

            // Daten in die Datei schreiben.
            channelBridge->WriteListEntrance2ToTextFile( listNum, fileNameTxt );
        }
    }

    iRes = channelBridge->writeEntrance2ListData( deviceAddress, number, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );

        if ( channelBridge->getLastErrorNumber( deviceAddress ) != IV_ERROR_BLOCK_TRANSFER_BREAK )
        {
            channelBridge->blockTransferDiscard( deviceAddress );
        }
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}
