/**
* @file  df_api.cpp
* @brief Zentrales Modul der DFComDLL.
*/
//////////////////////////////////////////////////////////////////////

#include "api/df_api.h"
#include "datafox_library_iv.h"
#include "library_version.h"

#if defined(_MSC_VER) || defined(__MINGW32__)
#define snprintf	    _snprintf
#endif

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/ini_parser.hpp>
#include <boost/foreach.hpp>
#include <string>
#include <set>
#include <exception>
#include <sstream>
#include <regex>

/*
"device.serial";
"device.type";
// Ermitteln der Hardware
"modul.board");
"modul.rs485";
"modul.tcpip";
"modul.modemMC";
"modul.rs485ZK";
"modul.transponder";
"modul.display";
"modul.keypad";
"modul.mmc";
"modul.gps";
"modul.usb";
"modul.radio";
"modul.dcf77";
"modul.inAnalog";
"modul.inDigital";
"modul.outDigital";
"modul.fingerprint";
"modul.externalReader";
"option.ioModule";
"option.tbAdaption");
"option.comServer";
"option.tpWrite";
"option.http";
"option.language";
"option.biokeyUpdate";
"option.setup";
"modul.touch";
"modul.sensor";
"modul.battery";
"modul.charge";	
*/

namespace {

class ini_struct
{
public:
    ini_struct()
    {
        // Alle Keys in Map halten
        setting.insert("serial");
        //    setting.insert("version");
        setting.insert("type");
        // Ermitteln der Hardware
        setting.insert("board");
        //    setting.insert("rs485");
        //    setting.insert("tcpip");
        //    setting.insert("modemMC");
        setting.insert("rs485ZK");
        setting.insert("transponder");
        //    setting.insert("display");
        //    setting.insert("keypad");
        //    setting.insert("mmc");
        //    setting.insert("gps");
        //    setting.insert("usb");
        //    setting.insert("radio");
        //    setting.insert("dcf77");
        //    setting.insert("inAnalog");
        //    setting.insert("inDigital");
        //    setting.insert("outDigital");
        //    setting.insert("fingerprint");
        setting.insert("externalReader");
        //    setting.insert("ioModule");
        //    setting.insert("tbAdaption");
        //    setting.insert("comServer");
        //    setting.insert("tpWrite");
        //    setting.insert("http");
        //    setting.insert("language");
        //    setting.insert("fingerprintUpdate");
        //    setting.insert("setup");
        //    setting.insert("touch");
        //    setting.insert("sensor");
        //    setting.insert("battery");
        //    setting.insert("charge");	
    }

public:
    std::set<std::string> setting;
};

}

// ########################################################
// #####################   EXPORTE    #####################
// ########################################################

#if defined(_MSC_VER) || defined(__MINGW32__)
// ########################################################
/**
* @fn    CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
* @brief Ermitteln der DLL - Version. Kompatible zu Windows - Standard.
*
* @param pdvi       { Zeiger auf Struktur zur Informationsuebergabe. }
*/
DF_API_EXTERN HRESULT DF_API_CALL DllGetVersion( DLLVERSIONINFO *pdvi )
{
    TCHAR szModuleName[MAX_PATH];
    DWORD dwHandle;
    DWORD dwLen;
    LPVOID lpBuffer = nullptr;
    UINT uiLen;

    unsigned char *pucData = nullptr;

    // zulaessige Bereiche pruefen
    if ( pdvi == nullptr )
    {
        return E_ABORT;
    }

    if ( pdvi->cbSize != sizeof(DLLVERSIONINFO) )
    {
        return E_ABORT;
    }

    // Modulname der DLL
    if ( !GetModuleFileName( theApp._hModule, szModuleName, MAX_PATH ) )
    {
        return E_ABORT;
    }

    // Laenge der Version - Resource
    dwLen = GetFileVersionInfoSize( szModuleName, &dwHandle );
    if ( !dwLen )
    {
        return E_ABORT;
    }

    pucData = new unsigned char[dwLen + 100]; // unicode
    // DLL - Versioninfo ermitteln
    if ( !GetFileVersionInfo( szModuleName, dwHandle, dwLen, (LPVOID)pucData ) )
    {
        delete pucData;
        return E_ABORT;
    }

    // Pointer auf allgemeine Infostruktur ermitteln
    if ( !VerQueryValue( (LPVOID)pucData, TEXT("\\\0"), &lpBuffer, &uiLen ) )
    {
        delete pucData;
        return E_ABORT;
    }

    // Daten zurueckgeben
    VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;
    pdvi->dwMajorVersion = pvs_ffi->dwFileVersionMS >> 16;
    pdvi->dwMinorVersion = pvs_ffi->dwFileVersionMS & 0x0000FFFF;
    pdvi->dwBuildNumber  = pvs_ffi->dwFileVersionLS >> 16;
    pdvi->dwPlatformID   = DLLVER_PLATFORM_WINDOWS;

    delete pucData;
    return NOERROR;
}
#endif

DF_API_EXTERN_DECORATE int DF_API_CALL DFCVersionLibrary( char *value, int valueLength )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *channelBridge = theApp.channelBridge( id );
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr) 
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s", __func__ );

    // zulaessige Bereiche pruefen
    if ( value == nullptr || valueLength < 12 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t%p\t%i\t(0, %i)", value, valueLength, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    strncpy( value, libraryVersion, valueLength - 1 );

    channelBridge->logEnd( "\t%s\t%i\t(1, %i)", value, valueLength, channelBridge->getLastErrorNumber( 0 ) );

    return 1;
}

DF_API_EXTERN_DECORATE void DF_API_CALL DFCGetErrorText( int connectionId, int errorNumber, int languageId, char *value, int valueLength )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, errorNumber, languageId );

    // zulaessige Bereiche pruefen
    if ( value == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t\t%i", valueLength );
        return;
    }

    switch ( errorNumber )
    {
    case IV_ERROR_CRC_PACKET:
        snprintf( value, valueLength, "CRC Fehler." );
        break;

    case IV_ERROR_CHANNEL_OPEN:
        snprintf( value, valueLength, "Oeffnen der Schnittstelle fehlgeschlagen." );
        break;

    case IV_ERROR_CHANNEL_CLOSED:
        snprintf( value, valueLength, "Schnittstelle ist geschlossen." );
        break;

    case IV_ERROR_CHANNEL_COMMUNICATION:
        snprintf( value, valueLength, "Allgemeiner Fehler waehrend Kommunikation." );
        break;

    case IV_ERROR_RECEIVE:
        snprintf( value, valueLength, "Geraet antwortet nicht." );
        break;

    case IV_ERROR_TRANSMIT:
        snprintf( value, valueLength, "Fehler waehrend Senden eingetreten." );
        break;

        /*
        case ERROR_AT_RECIEVING	:
        snprintf( szText, iLength, "Fehler waehrend Empfang eingetreten." );
        break;
        */

    case IV_ERROR_ADDRESS:
        snprintf( value, valueLength, "Antwort von falschem Geraet oder Antwort nicht sinnvoll." );
        break;

    case IV_ERROR_DEVICE_ID_REMOTE:
        snprintf( value, valueLength, "Keine gueltige Busnummer. (1 - 31, 254)" );
        break;

    case IV_ERROR_CHANNEL:
        snprintf( value, valueLength, "Kanal nicht vorhanden oder nicht erstellbar." );
        break;

    case IV_ERROR_FILE_OPEN:
        snprintf( value, valueLength, "Oeffnen einer Datei gescheitert." );
        break;

    case IV_ERROR_FILE_FORMAT:
        snprintf( value, valueLength, "z.B. falsches Format einer Setupdatei." );
        break;

    case IV_ERROR_LIST_PROCESSING:
        snprintf( value, valueLength, "Fehler bei Listendatenverarbeitung." );
        break;

    case IV_ERROR_VERSION_FIRMWARE:
        snprintf( value, valueLength, "Firmwareversion nicht kompatible zu Routine." );
        break;

    case IV_ERROR_DATA_AVAILABLE:
        snprintf( value, valueLength, "Es sind noch Daten im Geraet vorhanden." );
        break;

    case IV_ERROR_USER_CANCEL:
        snprintf( value, valueLength, "Durchfuehrung wurde von Anwender abgebrochen." );
        break;

    case IV_ERROR_DESCRIPTION_RECORD_UNLOADED:
        snprintf( value, valueLength, "Keine Datensatzbeschreibung geladen." );
        break;

    case IV_ERROR_DESCRIPTION_LIST_UNLOADED:
        snprintf( value, valueLength, "Keine Listenbeschreibung geladen." );
        break;

        /*
        case ERROR_FLASH_PROG:
        snprintf( szText, iLength, "Fehler beim Programmieren des Flash, Firmware." );
        break;
        */

    case IV_ERROR_FUNCTION_PARAMETER:
        snprintf( value, valueLength, "Parameter einer Funktion ausserhalb ihres Gueltigkeitsbereiches." );
        break;

    case IV_ERROR_VARIABLE_UNDEFINED:
        snprintf( value, valueLength, "Globale Variable nicht definiert." );
        break;

    case IV_ERROR_VARIABLE_SIZE:
        snprintf( value, valueLength, "Der verwendete Variablenwert ist zu lang." );
        break;

    case IV_ERROR_VARIABLE_TYPE:
        snprintf( value, valueLength, "Der verwendete Variablentype wuerde nicht eingehalten." );
        break;

    case IV_ERROR_VARIABLE_RIGHT:
        snprintf( value, valueLength, "Das Schreib-/Leserecht wuerde nicht eingehalten." );
        break;

    case IV_ERROR_VARIABLE_FORMAT_OR_RANGE:
        snprintf( value, valueLength, "Das verwendete Format oder der Wertebereich wuerde nicht eingehalten." );
        break;

        /*
        case ERROR_NO_DATA:
        snprintf( szText, iLength, "Anfrage obwohl keine Daten." );
        break;
        */

        /*
        case ERROR_MULTI:
        snprintf( szText, iLength, "Dublikat von Datensaetzen abgefangen." );
        break;
        */

    case IV_ERROR_BOOTVERSION:
        snprintf( value, valueLength, "Geraetedatei nicht mit Bootloaderversion kompatible." );
        break;

        /*
        case ERROR_BOOTFILEVERSION:
        snprintf( szText, iLength, "Firmwaredatei nicht mit Bootloaderversion kompatible." );
        break;
        */


    case IV_ERROR_FUNCTION_UNAVAILABLE:
        snprintf( value, valueLength, "Funktion steht im vorliegenden Fall nicht zur Verfuegung." );
        break;

    case IV_ERROR_PROCESSING:
        snprintf( value, valueLength, "Der Befehl wurde erfolgreich uebermittelt, konnte aber nicht durchgefuehrt werden." );
        break;

    case IV_ERROR_DEVICE_UNKNOWN:
        snprintf( value, valueLength, "Der Geraetetyp ist unbekannt. Update der Software erforderlich." );
        break;

    case IV_ERROR_DEVICEFILE_WRONG:
        snprintf( value, valueLength, "Die angegebene Geraetedatei stimmt nicht mit der vorliegenden Hardware ueberein." );
        break;

    case IV_ERROR_DEVICE_WRONG:
        snprintf( value, valueLength, "Geraetetyp stimmt nicht. z.B. Versuchtes Einspielen von Setupdatei fuer PZE-MasterIV in BDE-MasterIV." );
        break;

    case IV_ERROR_FILE_EXTENSION:
        snprintf( value, valueLength, "Die vorliegende Dateiendung wird nicht unterstuetzt." );
        break;

    case IV_ERROR_ENTRANCE2_OFF:
        snprintf( value, valueLength, "Die Zutrittskontrolle ist deaktiviert." );
        break;

    case IV_ERROR_NAME_UNKNOWN:
        snprintf(value, valueLength, "Der Tabellenname oder Spaltenname ist undefiniert." );
        break;

    case IV_ERROR_HANDLE:
        snprintf( value, valueLength, "Ein angegebener Handle ist nicht gueltig." );
        break;

    case IV_ERROR_NULL_RESULT:
        snprintf( value, valueLength, "Der anzuwendende Filter wuerde keine Datensaetze liefert." );
        break;

    case IV_ERROR_BOF:
        snprintf( value, valueLength, "Sprung fuehrte zum Anfang der Tabelle. (Base of File)" );
        break;

    case IV_ERROR_EOF:
        snprintf( value, valueLength, "Sprung fuehrte zum Ende der Tabelle. (End of File)" );
        break;

    case IV_ERROR_VALUE:
        snprintf( value, valueLength, "Der angegebene Feldwert ist Fehlerhaft." );
        break;

    case IV_ERROR_OVERFLOW:
        snprintf( value, valueLength, "Bei einer Datenuebertragung ist ein Speicherueberlauf eingetreten." );
        break;

    case IV_ERROR_NO_DATA_AVAILABLE:
        snprintf( value, valueLength, "Es liegen keine Daten vor." );
        break;

    case IV_ERROR_PROCESSING_DLL:
        snprintf( value, valueLength, "Der Befehl konnte zwar uebermittelt werden, bei den Daten liegt jedoch ein Fehler vor." );
        break;

    case IV_ERROR_MODULE_BUSY:
        snprintf( value, valueLength, "Das angesprochene Modul ist zur Zeit durch einen anderen Process blockiert." );
        break;

    case IV_ERROR_FILE_LOAD_XML:
        snprintf( value, valueLength, "Fehler beim Laden der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_UNKNOWN_XML:
        snprintf( value, valueLength, "Unbekannte Elemente der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_CONTENT_XML:
        snprintf( value, valueLength, "Fehler im Dateninhalt der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_MODIFY:
        snprintf( value, valueLength, "Fehler bei Uebernahme (Anwendung) der zu modifizierenden Daten" );
        break;

    case IV_ERROR_ALLOC_MEM:
        snprintf( value, valueLength, "Speicherallokation konnte nicht durchgefuehrt werden." );
        break;

    case IV_ERROR_VERSION_DEVICEFILE:
        snprintf( value, valueLength, "Diese Version der Geraetedatei kann aufgrund ihrer Aktualitaet nicht verarbeitet werden." );
        break;

    case IV_ERROR_FILE_PROCESSING:
        snprintf( value, valueLength, "Bei der Verarbeitung der Dateidaten trat ein Fehler auf." );
        break;

    case IV_ERROR_DEVICEFILE_MATCH:
        snprintf( value, valueLength, "Die vorliegende Geraetedatei passt nicht zur vorliegenden Hardware. Oder aber: Im Archiv wurde keine passende Geraetedatei gefunden." );
        break;

    case IV_ERROR_DEVICEFILE_LOAD:
        snprintf( value, valueLength, "Fehler beim Laden der Geraetedatei(archive) aufgetreten." );
        break;

    case IV_ERROR_AC_STARTED:
        snprintf( value, valueLength, "Die Aktive Verbindung ist gestartet." );
        break;

    case IV_ERROR_AC_STOPPED:
        snprintf( value, valueLength, "Die Aktive Verbindung ist nicht gestartet." );
        break;

    case IV_ERROR_AC_PORTINUSE:
        snprintf( value, valueLength, "Der angegebene Listensocket ist bereits in Verwendung." );
        break;

    case IV_ERROR_AC_NO_NOTIFY_RECORD:
        snprintf( value, valueLength, "Die Aktive Meldung von Datensaetzen ist deaktiviert." );
        break;

    case IV_ERROR_AC_LISTENER:
        snprintf( value, valueLength, "Allgemeiner Fehler beim Starten der Aktiven Verbindung." );
        break;

    case IV_ERROR_NEED_PASSWORD:
        snprintf( value, valueLength, "Das angegebene Passwort stimmt nicht mit dem benoetigten ueberein. Es wird ein Passwort gefordert." );
        break;

    case IV_ERROR_PASSWORD_LOCK:
        snprintf( value, valueLength, "Geraet ist aufgrund von fehlerhaften Passworteingaben fuer 60 Sekunden gesperrt." );
        break;

    case IV_ERROR_FILE_WRITE:
        snprintf( value, valueLength, "Das Schreiben in die Datei ist fehlgeschlagen." );
        break;

    case IV_ERROR_FILE_READ:
        snprintf( value, valueLength, "Das Lesen aus der Datei ist fehlgeschlagen." );
        break;

    case IV_ERROR_FILE_CRC:
        snprintf( value, valueLength, "Fehlerhafter Dateninhalt. Die Pruefung auf die Datenkonsistenz per CRC schlug fehl." );
        break;

    case IV_ERROR_MEMORY_FULL:
        snprintf( value, valueLength, "Es steht kein weiterer Speicherplatz zur Verfuegung.\nZ. B. kann die maximale aufnehmbare Anzahl Fingertemplates erreicht sein." );
        break;

    case IV_ERROR_FILE_DATA_FORMAT:
        snprintf( value, valueLength, "Die in der Datei vorliegenden Daten stimmen nicht mit den geforderten ueberein.\nZ. B. in der Datei liegen Fingertemplates vom Typ Idencom-Compact vor, es werden jedoch DIN Erfordert." );
        break;

    case IV_ERROR_NOT_ENOUGH_FLASH:
        snprintf( value, valueLength, "Es steht nicht genuegend Flash-Speicher zur Verfuegung, um die Daten aufzunehmen." );
        break;

    case IV_ERROR_NO_MODULE:
        snprintf( value, valueLength, "Das von Ihnen angesprochene Modul ist eine Optionale Hardware-Erweiterung und ist im vorliegenden Geraet nicht verfuegbar." );
        break;

    case IV_ERROR_VERSION_SETUPFILE:
        snprintf( value, valueLength, "In der Setupdatei werden Erweiterungen verwendet, die einen aktuelleren Firmwarestand voraussetzen." );
        break;

    case IV_ERROR_WRONG_FIRMWARE:
        snprintf( value, valueLength, "Die abzuarbeitende Aufgabe wird durch die vorliegenden Firmwareversion nicht unterstuetzt." );
        break;

    case IV_ERROR_WRONG_CONFIG:
        snprintf( value, valueLength, "Die abzuarbeitende Aufgabe wird durch die vorliegenden Geraetekonfiguration nicht unterstuetzt." );
        break;

    case IV_ERROR_TIMEBOY_I_II_III:
        snprintf( value, valueLength, "Bei dem angesprochenen Timeboy handelt es sich um kein Model der vierer Reihe." );
        break;

    case IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der DFComDLL voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_LIBRARY_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte Soll-Version der DFComDLL vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.01.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.01.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.01.xx vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.02.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.02.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.02.xx vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.03.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.03.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.03.xx vor." );
        break;

    case IV_ERROR_FINGERPRINT:
        snprintf( value, valueLength, "Das Fingerprintmodul bringt einen nicht naeher spezifizierten Fehler." );
        break;

    case IV_ERROR_FINGERPRINT_MEMORY_FULL:
        snprintf( value, valueLength, "Die Datenbank des Fingerprintmoduls ist voll." );

        break;
    case IV_ERROR_FINGERPRINT_TEMPLATES_NONE:
        snprintf( value, valueLength, "Das Fingerprintmodul enthaelt keine Fingertemplates." );
        break;

    case IV_ERROR_FINGERPRINT_TEMPLATE_EXISTS:
        snprintf( value, valueLength, "Das Fingertemplate existiert bereits." );
        break;

    case IV_ERROR_FINGERPRINT_TEMPLATE_NOT_EXISTS:
        snprintf( value, valueLength, "Das angeforderte Fingertemplate existiert nicht." );
        break;

     default:
        snprintf( value, valueLength, "Unbekannter Fehler." );
        break;
    }

    channelBridge->logEnd( "\t%s\t%i", value, valueLength );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckDevice( int connectionId, int deviceAddress, int *errorNumber, int devicePollRetry )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || errorNumber == nullptr || devicePollRetry < 1 )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t%i\t(0, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, devicePollRetry, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t%i\t(0, %i)", *errorNumber, devicePollRetry, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->checkDevice( deviceAddress, errorNumber, devicePollRetry );
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
    }

    channelBridge->logEnd( "\t%i\t%i\t(%i, %i)", *errorNumber, devicePollRetry, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetDevicePollRetry( int connectionId )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i", __func__, connectionId );
    channelBridge->logEnd( "\t(%i, %i)", channelBridge->devicePollRetry_, channelBridge->getLastErrorNumber( 0 ) );

    return channelBridge->devicePollRetry_;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSendMessage( int connectionId, int deviceAddress, unsigned char visibleTime, unsigned char doAction, unsigned char audibleSignal, const char *text, int textLength )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i\t%i\t%s\t%i", __func__,
        connectionId, deviceAddress, visibleTime, doAction, audibleSignal,
        (text == nullptr) ? "" : text, textLength );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false 
        || (doAction != 0 && doAction != 1) 
        || textLength < 0 
        || (doAction == 0 && textLength > 242)
        || (doAction == 1 && textLength > (IV_PACKET_LENGTH_DATA - (5 + 3)))
        || text == nullptr )
    {
        if (channelBridge->isValidDeviceAddress( deviceAddress ) == false)
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    std::string lText( text, textLength );
    int iRes = channelBridge->sendMessage( deviceAddress, visibleTime, doAction, audibleSignal, lText, &iError );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSendInfotext( int connectionId, int deviceAddress, const char *text, int textLength )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%s\t%i", __func__,
        connectionId, deviceAddress, (text == nullptr) ? "" : text, textLength );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || textLength < 0 || textLength > 242 || text == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    std::string lText( text, textLength );
    int iRes = channelBridge->sendInfotext( deviceAddress, lText, &iError );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    return iRes;
}


DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSetTime( int connectionId, int deviceAddress, const unsigned char *dateTimeBuffer )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = channelBridge->setDateTime( deviceAddress, dateTimeBuffer, &iError );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    // Uhrzeit mitloggen
    if ( dateTimeBuffer != nullptr )
    {
        channelBridge->logEnd( "\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i, %i)",
            dateTimeBuffer[0], dateTimeBuffer[1], dateTimeBuffer[2], dateTimeBuffer[3],
            dateTimeBuffer[4], dateTimeBuffer[5], dateTimeBuffer[6], 
            iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    }
    else
    {
        channelBridge->logEnd( "\t\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComGetTime( int connectionId, int deviceAddress, unsigned char *dateTimeBuffer )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || dateTimeBuffer == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t\t(0, %i)", channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = channelBridge->getDateTime( deviceAddress, dateTimeBuffer, &iError );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    if ( iRes )
    {
        channelBridge->logEnd( "\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i, %i)",
            dateTimeBuffer[0], dateTimeBuffer[1], dateTimeBuffer[2], dateTimeBuffer[3],
            dateTimeBuffer[4], dateTimeBuffer[5], dateTimeBuffer[6], 
            iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    }
    else
    {
        channelBridge->logEnd( "\t\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComVersion( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getProtocolVersion2( deviceAddress, TRUE, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(0x%04X, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    // Kommunikationsversion z.B: 0x00000103
    // Die Versionsnummer ist ein 16Bit - Wert
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetVersionFirmware( int connectionId, int deviceAddress, char *value, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || value == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    std::string lVersion;
    int iRes = channelBridge->getVersionFirmware( deviceAddress, lVersion );
    if ( iRes != IV_ERROR_SUCCESS )
    {
        iRes = channelBridge->resultToError( iRes, errorNumber );
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        //channelBridge->setLastErrorNumber( deviceAddress, (iRes < 0) ? -iRes : IV_ERROR_VERSION_FIRMWARE );
        //iRes = 0;
    }
    else
    {
        lVersion.copy( value, lVersion.size() );
        value[lVersion.size()] = 0;
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", (iRes) ? value : "", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetVersionFirmwareFromFile( int connectionId, const char *filePath, char *value, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i", __func__, connectionId );

    // zulaessige Bereiche pruefen
    if ( filePath == nullptr || value == nullptr || errorNumber == nullptr || strlen( filePath ) < 1 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( 0 );
        }

        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;
    std::string version;

    int iRes = channelBridge->getVersionFirmwareFromFile( filePath, version, errorNumber );
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( 0, *errorNumber );
    }

    std::strcpy( value, version.c_str() );
    channelBridge->logEnd( "\t%s\t%s\t%i\t(%i, %i)", filePath, (iRes) ? value : "", *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetSeriennummer( int connectionId, int deviceAddress, int *errorNumber, int *value )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || errorNumber == nullptr || value == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getSerialNum( deviceAddress, value, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t%i\t(%i, %i)", *errorNumber, *value, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCStartTransparencyMode( int connectionId, int deviceAddress, int id, int flags, unsigned int sizeBufferRead, unsigned int sizeBufferWrite, int timeout, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, id, flags, sizeBufferRead, sizeBufferWrite );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->startTransparencyMode( deviceAddress, id, flags, sizeBufferRead, sizeBufferWrite, timeout, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCStopTransparencyMode( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->stopTransparencyMode( deviceAddress, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCIsTransparencyMode( int connectionId )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i", __func__, connectionId );

    int iRes = channelBridge->isTransparencyMode();

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCPing( int connectionId, int deviceAddress, int attempts, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, attempts );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || attempts < 1 || attempts > 1000 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->ping( deviceAddress, attempts, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE void DF_API_CALL DFCSetCallBack( int connectionId, int (*functionPointer)() )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t", __func__, connectionId );

    channelBridge->setCallBackFunction( functionPointer );

    channelBridge->logEnd( "" );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetGlobVar( int connectionId, int deviceAddress, const char *name, int nameType, const char *value, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || name == nullptr || value == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t\t%i\t\t%i\t(%i, %i)", nameType, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich pruefen.
    switch ( nameType )
    {
    case 0: // Ansprechen ueber Nummer
        if ( *name < 1 || *name > 8 )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        channelBridge->logAppend( "\t%i\t%i\t%s", *name, nameType, (char*)value );
        break;

    case 1: // Ansprechen ueber Name
        if ( (strlen( (const char*)name ) < 1) || (strlen( (const char*)name ) > 17) )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        channelBridge->logAppend( "\t%s\t%i\t%s", (char*)name, nameType, (char*)value );
        break;

    case 2: // Ansprechen ueber Systemname
        if ( strlen( (const char*)name ) < 1 )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        // Bereichspruefung der Systemvariablen
        if ( (strstr( (char*)name, "SERVICE.DBLINPUT" ) && (atoi( (char*)value ) < 0 || atoi( (char*)value ) > 65499)) ||
            (strstr( (char*)name, "SERVICE.DEMANDTIMER" ) && (atoi( (char*)value ) < 0 || atoi( (char*)value ) > 255)) )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }
        channelBridge->logAppend( "\t%s\t%i\t%s", (char*)name, nameType, (char*)value );
        break;

    default:
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
    }

    if ( channelBridge->getLastErrorNumber( deviceAddress ) != IV_ERROR_SUCCESS )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        if ( nameType == 0 )
        {
            channelBridge->logEnd( "\t%i\t%i\t\t%i\t(%i, %i)", *name, nameType, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        }
        else
        {
            channelBridge->logEnd( "\t%s\t%i\t\t%i\t(%i, %i)", (char*)name, nameType, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        }

        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->setGlobVar( deviceAddress, name, nameType, (char*)value, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetGlobVar( int connectionId, int deviceAddress, const char *name, int nameType, char *value, int valueLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || name == nullptr || value == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t\t%i\t\t%i\t%i\t(%i, %i)", nameType, valueLength, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich pruefen.
    switch ( nameType )
    {
    case 0: // Ansprechen ueber Nummer
        if ( *name < 1 || *name > 8 )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        channelBridge->logAppend( "\t%i\t%i", *name, nameType );
        break;

    case 1: // Ansprechen ueber Name
        if ( (strlen( (const char*)name ) < 1) || (strlen( (const char*)name ) > 17) )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        channelBridge->logAppend( "\t%s\t%i", (char*)name, nameType );
        break;

    case 2: // Ansprechen ueber Systemname
        if ( strlen( (const char*)name ) < 1 )
        {
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
            break;
        }

        channelBridge->logAppend( "\t%s\t%i", (char*)name, nameType );
        break;

    default:
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
        break;
    }

    if ( channelBridge->getLastErrorNumber( deviceAddress ) != IV_ERROR_SUCCESS )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        if ( nameType == 0 )
        {
            channelBridge->logEnd( "\t%i\t%i\t\t%i\t%i\t(%i, %i)", *name, nameType, valueLength, channelBridge->getLastErrorNumber( deviceAddress ), 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        }
        else
        {
            channelBridge->logEnd( "\t%s\t%i\t\t%i\t%i\t(%i, %i)", (char*)name, nameType, valueLength, channelBridge->getLastErrorNumber( deviceAddress ), 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        }

        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t\t%i\t%i\t(%i, %i)", valueLength, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getGlobVar( deviceAddress, name, nameType, (char*)value, valueLength, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", (char*)value, valueLength, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCloseRelay( int connectionId, int deviceAddress, int number, int timeout, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number, timeout );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || number < 0 || number > 16 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->closeRelay( deviceAddress, number, timeout, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetRelayState( int connectionId, int deviceAddress, int number, int *state, int *timeout, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || number < 0 || number > 16 || state == nullptr || timeout == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getRelayState( deviceAddress, number, state, timeout, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t%i\t%i\t(%i, %i)", *state, *timeout, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCOpenRelay( int connectionId, int deviceAddress, int number, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || number < 0 || number > 16 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->openRelay( deviceAddress, number, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetInfo( int connectionId, int deviceAddress, const char *name, const char *params, char *text, int *textLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false 
        || name == nullptr || params == nullptr || text == nullptr || textLength == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", (name == nullptr) ? "" : name, (params == nullptr) ? "" : params,
            (text == nullptr) ? "" : text, (textLength == nullptr) ? 0 : *textLength, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );

        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", name, params, text, *textLength, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    std::string info;
    int iRes = channelBridge->getInfo( deviceAddress, name, params, info );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        *textLength = static_cast<int>( info.copy( text, *textLength - 1 ) ) + 1;
        text[*textLength - 1] = 0; // Nullterminierung
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 7:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_MODULE_BUSY );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", name, params, text, *textLength, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCOpenComServerMode( int connectionId, int deviceAddress, const char *comm, int baudRate, int byteSize, int parity, int stopBits, int flags, int timeoutToClose, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen // HK erweitert um 57600 und 115200
    if ( deviceAddress != 254 || comm == nullptr
        || (baudRate != 9600 && baudRate != 19200 && baudRate != 38400 && baudRate != 57600 && baudRate != 115200)
        || flags & 0xFFFF0000 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i, %i)", 
            (comm == nullptr) ? "" : comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );

        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = channelBridge->openComServerMode( deviceAddress, comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 1: // Es wurden zu wenige Paramteter angegeben.
            case 2: // Es wurden zu viele Paramteter angegeben.
            case 3: // Die Angabe des comm Parameters wird nicht unterstuetzt.
            case 4: // Die Angabe des baudRate Parameters wird nicht unterstuetzt.
            case 5: // Die Angabe des byteSize Parameters wird nicht unterstuetzt.
            case 6: // Die Angabe des parity Paramters wird nicht unterstuetzt.
            case 7: // Die Angabe des stopBits Paramters wird nicht unterstuetzt.
            case 8: // Die Angabe des flags Parameters wird nicht unterstuetzt.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
                break;

            case 28: // Passwortangabe fehlerhaft
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29: // 3 malige flascheingabe, Geraet geblockt.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i, %i)", comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCloseComServerMode( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( deviceAddress != 254 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Eine Sekunde Pause, fuer Pufferleerung.
    std::this_thread::sleep_for( std::chrono::seconds( 1 ) );

    int iRes = channelBridge->closeComServerMode( deviceAddress );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 1:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_OVERFLOW );
                break;

            case 28: // Passwort fehlerhaft
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29: // 3 malige flascheingabe, Geraet geblockt.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

/*
mask:
0x00000001 External Reader options 0=GIS, 1=PHG
0x00000002 IOModul
0x00000004 TBAdaption
0x00000008 ComServer
0x00000010 TransponderWrite
0x00000020 UpdateBiokey
0x00000040 Http-Protokoll
0x00000080 Signalverarbeitung
0x00000100 Setup und Listen beibehalten.

options: zustand 0 ist off, und 1 on
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetOptionFirmware( int connectionId, int deviceAddress, int mask, int options, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, mask, options );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || mask < 0 || mask > 0x100 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->setOptionFirmware( deviceAddress, mask, options );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetOptionFirmware( int connectionId, int deviceAddress, int *mask, int *options, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || mask == nullptr || options == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t%i\t%i\t(%i, %i)", (mask == nullptr) ? 0 : mask, (options == nullptr) ? 0 : options, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t%i\t%i\t(%i, %i)", mask, options, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getOptionFirmware( deviceAddress, *mask, *options );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE);
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t%i\t%i\t(%i, %i)", mask, options, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetFontType( int connectionId, int deviceAddress, int type, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, type );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || type < 0 || type > 6 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->setTypeFont( deviceAddress, type, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCPressVirtualKey( int connectionId, int deviceAddress, int keyCode, int flags, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, keyCode, flags );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false 
        || keyCode < 0x70 || (keyCode > 0x7E && keyCode != 0xEE) || flags > 1 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->pressVirtualKey( deviceAddress, keyCode, flags, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber  );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetPasswordKey( int connectionId, int deviceAddress, char *value, int *valueLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || value == nullptr || valueLength == nullptr || *valueLength < 17 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", (value == nullptr) ? "" : value, (valueLength == nullptr) ? 0 : *valueLength, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", value, *valueLength, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getPasswordKey( deviceAddress, value );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *valueLength = (int)strlen( value );
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        } 
        else
        {
            // Funktionsfehler auswerten.
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", value, *valueLength, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetPassword( int connectionId, const char *value, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i", __func__, connectionId );

    // zulaessige Bereiche pruefen
    if ( value == nullptr || strlen( value ) > 16 || errorNumber == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( 0 );
        }

        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", (value == nullptr) ? "" : "****", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    channelBridge->setPassword( value );

    channelBridge->logEnd( "\t****\t%i\t(%i, %i)", *errorNumber, 1, channelBridge->getLastErrorNumber( 0 ) );

    return 1;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCReset( int connectionId, int deviceAddress, int mode, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, mode );

    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || errorNumber == nullptr 
        || (mode != 1 && mode != 2 && mode != 3 && mode != 6 && mode != 7 && mode != 100 && mode != 101) )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t(0, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    iRes = channelBridge->reset( deviceAddress, mode, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetFlashStatus( int connectionId, int deviceAddress, int *value, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || value == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t%i\t(%i, %i)", *value, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->getFlashStatus( deviceAddress, value, errorNumber );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, *errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t%i\t(%i, %i)", *value, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetCommunicationPassword( int connectionId, const char *value, int valueLength, int sessionTimeout, int *errorNumber )
{
    // Auf Logchannel ausweichen, wenn die Passwortangabe fuer alle gueltig sein soll.
    int id = connectionId;
    if ( connectionId == -1 )
    {
        id = LOG_CHANNEL_ID;
    }

    ChannelBridge *channelBridge = theApp.channelBridge( id );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i", __func__, connectionId );

    // zulaessige Bereiche pruefen
    if ( value == nullptr || valueLength < 0 || valueLength > 16
        || sessionTimeout < 0 || (sessionTimeout > 0 && sessionTimeout < 60) || sessionTimeout > 100000 || errorNumber == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( 0 );
        }

        channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", (value == nullptr) ? "" : "****", valueLength, sessionTimeout, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    int res = theApp.SetCommunicationPassword( connectionId, value, valueLength, sessionTimeout );
    if ( res == 0 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_CHANNEL );
        *errorNumber = channelBridge->getLastErrorNumber( 0 );
    }

    channelBridge->logEnd( "\t****\t%i\t%i\t%i\t(%i, %i)", valueLength, sessionTimeout, *errorNumber, res, channelBridge->getLastErrorNumber( 0 ) );

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCReadHardwareInfo( int connectionId, int deviceAddress, char *value, int *valueLength, int flags, int *errorNumber ) 
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "\t%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || value == nullptr || valueLength == nullptr || flags != 0 || errorNumber == nullptr )
    {
        if (channelBridge->isValidDeviceAddress( deviceAddress ) == false)
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", (value == nullptr) ? "" : value, (valueLength == nullptr) ? 0 : *valueLength, 
            flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );

        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", value, *valueLength, flags, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int res = 0;
    unsigned char ucVal;
    unsigned int uiVal;
    unsigned short usVal;
    enum
    {
        ucType, 
        uiType, 
        usType
    } putType;

    // Klasse fuer die Verwaltung der Daten der Hardwareinfo
    ini_struct is;
    // Leeres PropertyTree Objekt
    using boost::property_tree::ptree;
    ptree pt;
    int readAll = 0;

    // Laden der vorliegenden Ini-Datei.
    // Ausnahme wird ausgeloest, wenn ein Fehler vorliegt.
    try
    {
        //read_ini(fileName, pt);
        // Aktuell werden noch alle gelesen
        readAll = 1;
    }
    catch ( boost::property_tree::ini_parser::ini_parser_error &e )
    {
        // Oeffnen der Datei fehlgeschlagen?
        if ( e.message() != "cannot open file" )
        {
            //TRACE("Fehlernachricht: %s\n", e.message().c_str());
        }

        // Es werden alle moeglichen Informationen ermittelt.
        readAll = 1;
    }

    HardwareInfo hardwareInfo;
    if ( (res = channelBridge->getHardwareInfo( deviceAddress, hardwareInfo )) != IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_UNREACHABLE );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        res = 0;
    }
    else
    {
        // Aktuell auf die "Vorgängerversion" mappen.
        StructHardwareInfo hardwareInfoOld( hardwareInfo.exportStructHardwareInfo() );

        std::set<std::string>::iterator i;
        i = is.setting.begin();
        for ( ; i != is.setting.end(); ++i )
        {

            if ( readAll == 0 && pt.get_child_optional( *i ) == boost::none ) // Dieser Key soll nicht gelesen werden.
                continue;

            res = IV_ERROR_SUCCESS;
            putType = ucType;

            if ( *i == "serial" )
            {
                if ( (res = channelBridge->channel4_.read_serial_number( deviceAddress, uiVal )) < IV_ERROR_SUCCESS ) 
                    break;

                putType = uiType;
            }
            else if ( *i == "version" )
            {
                // Version der Hardwareinfo ermitteln
                ucVal = hardwareInfoOld.version;
            }
            else if ( *i == "type" )
            {
                // Geraetetyp ermitteln
                ucVal = hardwareInfoOld.typeDevice;
            }
            else if ( *i == "board" )
            {
                // Boardtyp ermitteln
                ucVal = hardwareInfoOld.typeBoard;
            }
            else if ( *i == "rs485" )
            {
                // RS485 ermitteln
                ucVal = hardwareInfoOld.componentRS485_0;
            }
            else if ( *i == "tcpip" )
            {
                // TCPIP ermitteln
                ucVal = hardwareInfoOld.typeTCPIP;
            }
            else if ( *i == "modemMC" )
            {
                // ModemMC35i ermitteln
                ucVal = hardwareInfoOld.typeModemMC;
            }
            else if ( *i == "rs485ZK" )
            {
                // RS485 (intern) ermitteln
                ucVal = hardwareInfoOld.componentRS485_1;
            }
            else if ( *i == "transponder" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeTransponder;
            }
            else if ( *i == "display" )
            {
                // Typ ermitteln (erst ab 04.01.01.p / Bootloader 4.0)
                ucVal = hardwareInfoOld.typeDisplay;
            }
            else if ( *i == "keypad" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeKeypad;
            }
            else if ( *i == "mmc" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeMMC;
            }
            else if ( *i == "gps" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeGPS;
            }
            else if( *i == "usb" )
            {
                // Komponende ermitteln
                ucVal = hardwareInfoOld.componentUSB;
            }
            else if ( *i == "radio" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeRadio;
            }
            else if ( *i == "dcf77" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeDCF77;
            }
            else if ( *i == "inAnalog" )
            {
                // Komponende ermitteln
                uiVal = hardwareInfoOld.componentInAnalog;

                putType = uiType;
            }
            else if ( *i == "inDigital" )
            {
                // Komponende ermitteln
                uiVal = hardwareInfoOld.componentInDigital;

                putType = uiType;
            }
            else if ( *i == "outDigital" )
            {
                // Komponende ermitteln
                uiVal = hardwareInfoOld.componentOutDigital;

                putType = uiType;
            }
            else if ( *i == "fingerprint" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeFingerprint;
            }
            else if ( *i == "externalReader" )
            {
                // Typ ermitteln
                if ((res = channelBridge->channel4_.read_external_transponder_reader_type(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "ioModule" )
            {
                // Option ermitteln
                if ((res = channelBridge->channel4_.read_io_module_option(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "tbAdaption" )
            {
                // Option ermitteln
                if ((res = channelBridge->channel4_.read_timeboy_adaption_option(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "comServer" )
            {
                // Option ermitteln
                if ((res = channelBridge->channel4_.read_com_server_option(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "tpWrite" )
            {
                // Option ermitteln
                if ((res = channelBridge->channel4_.read_transponder_write_option(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "http" )
            {
                // Option ermitteln
                if ( (res = channelBridge->channel4_.read_http_option( deviceAddress, ucVal )) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "language" )
            {
                // Option ermitteln
                if ( (res = channelBridge->channel4_.read_localization_option( deviceAddress, ucVal )) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "fingerprintUpdate" )
            {
                // Option ermitteln
                if ( (res = channelBridge->channel4_.read_fingerprint_scanner_update_option( deviceAddress, ucVal )) < IV_ERROR_SUCCESS)
                    break;
            }
            else if ( *i == "setup" )
            {
                unsigned short mask = 0;
                // Option ermitteln
                if ( (res = channelBridge->channel4_.read_setup_options( deviceAddress, mask, usVal )) < IV_ERROR_SUCCESS)
                    break;

                putType = usType;
            }
            else if ( *i == "touch" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeTouch;
            }
            else if ( *i == "sensor" )
            {
                // Typ ermitteln
                ucVal = hardwareInfoOld.typeSensor;
            }
            else if ( *i == "battery" )
            {
                // Komponente ermitteln
                ucVal = hardwareInfoOld.componentBattery;
            }
            else if( *i == "charge" )
            {
                // Komponente ermitteln
                ucVal = hardwareInfoOld.componentCharge;
            }
            else
            {
                // Kein bekannter Eintrag.
                continue;
            }

            if ( res == IV_ERROR_SUCCESS ) 
            {
                switch ( putType )
                {
                case uiType:
                    pt.put( *i, uiVal );
                    break;

                case usType:
                    pt.put( *i, usVal );
                    break;

                default:
                    pt.put( *i, ucVal );
                }

                //TRACE("Key: %s\n", (*i).c_str());
            }
        }

        try
        {
            // Write property tree to INI file
            std::ostringstream os;
            write_ini( os, pt );
            os << std::ends;
            strncpy( value, os.str().data(), *valueLength - 1 );
            *valueLength = static_cast<int>( strlen( value ) );
            res = 1;
        }
        catch( std::exception &e )
        {
            (void)e;
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FILE_WRITE );
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
            res = 0;
        }
    }


    channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", value, *valueLength, flags, *errorNumber, res, channelBridge->getLastErrorNumber( deviceAddress ) );
    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCRecordVolume( int connectionId, int deviceAddress, int *recordCount, int *memoryUsage, int *memorySize, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || recordCount == nullptr || memoryUsage == nullptr || memorySize == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%i\t%i\t%i\t%i\t(%i, %i)", (recordCount == nullptr) ? 0 : *recordCount, (memoryUsage == nullptr) ? 0 : *memoryUsage, (memorySize == nullptr) ? 0 : *memorySize, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%i\t%i\t%i\t%i\t(%i, %i)", *recordCount, *memoryUsage, *memorySize, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->recordVolume( deviceAddress, *recordCount, *memoryUsage, *memorySize );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        } 
        else
        {
            // Funktionsfehler auswerten.
            channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t%i\t%i\t%i\t(%i, %i)", *recordCount, *memoryUsage, *memorySize, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckAE( int connectionId, int deviceAddress )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = channelBridge->checkDevice( deviceAddress, &iError, 1 );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE void* DF_API_CALL DFCGetClass( int connectionId, int classId )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return nullptr;
    }

    switch ( classId )
    {
    case 0:
        return channelBridge;

    case 1:
    case 2:
        return &channelBridge->channel1_2_;

    case 4:
        return &channelBridge->channel4_;

    default:
        return nullptr;
    }

    return nullptr;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCommandInspectionStation( int connectionId, int deviceAddress, int requestAddress, char *requestParams, int requestParamsSize, int *answerAddress, char *answerParams, int *answerParamsSize, int secondsToWaitForAnswer )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    // Parameter pruefen
    if (secondsToWaitForAnswer < 1 )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    return channelBridge->channel4_.inspection_station_command( deviceAddress, requestAddress, requestParams, requestParamsSize, (unsigned char&)*answerAddress, answerParams, (int&)*answerParamsSize, secondsToWaitForAnswer );
}

#ifdef _DEBUG

DF_API_EXTERN_DECORATE int DF_API_CALL DFCWatchdogReset( int connectionId, int deviceAddress )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    iRes = channelBridge->restart( deviceAddress, &iError );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, iError );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

#endif //_DEBUG

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCustomCommand( int connectionId, int deviceAddress, int identification, int customCommand, const char *dataIn, int sizeIn, char *dataOut, int *sizeOut, int *errorNumber )
{
	ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
	// Wurde Channel-Objekt ermittelt.
	if ( channelBridge == nullptr )
	{
		if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

		return 0;
	}

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

	channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, customCommand );

	// zulaessige Bereiche pruefen
	if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
		|| customCommand < 0
		|| dataIn == nullptr || dataOut == nullptr || sizeOut == nullptr
		|| errorNumber == nullptr )
	{
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%i\t%s\t%i\t%i\t(%i, %i)", (dataIn == nullptr) ? "" : dataIn, sizeIn, (dataOut == nullptr) ? "" : dataOut, (sizeOut == nullptr) ? 0 : *sizeOut, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
		return 0;
	}

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        
        channelBridge->logEnd( "\t%s\t%i\t%s\t%i\t%i\t(%i, %i)", dataIn, sizeIn, dataOut, *sizeOut, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
		return 0;
	}

	int iRes = channelBridge->customCommand( deviceAddress, identification, customCommand, dataIn, sizeIn, dataOut, sizeOut );
	if ( iRes == IV_ERROR_SUCCESS )
	{
		// Kein Fehler
		iRes = 1;
	}
	else if ( iRes > IV_ERROR_SUCCESS )
	{
		// Fehler bei Durchfuehrung des Befehls
		// Modulfehler auswerten.
		if ( (iRes >> 8) & 0xFF )
		{
			if ( ((iRes >> 8) & 0xFF) == 2 )
			{
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
			}
			else
			{
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
			}
		}
		else
		{
			// Funktionsfehler auswerten.
			switch ( iRes & 0xFF )
			{
			case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
				break;

			case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
				break;

			default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
			}
		}

		iRes = 0;
	}
	else
	{
		// Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
		iRes = 0;
	}

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress  );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%s\t%i\t%i\t(%i, %i)", dataIn, sizeIn, dataOut, *sizeOut, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

	return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetLastErrorNumber( int connectionId, int deviceAddress )
{
    int res = IV_ERROR_SUCCESS;
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return IV_ERROR_CHANNEL;
    }

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        res = IV_ERROR_FUNCTION_PARAMETER;
        channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    res = channelBridge->getLastErrorNumber( deviceAddress );

    channelBridge->logEnd("\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );

	return res;
}

DF_API_EXTERN int DF_API_CALL DFCBlockTransferSetDuration( int connectionId, int deviceAddress, int blockTypesAllowed, int duration )
{
    int res = 0;
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return res;
    }

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, duration );

    // zulaessige Bereiche pruefen
    if ( (deviceAddress != -1 && channelBridge->isValidDeviceAddress(deviceAddress) == false)
        || (blockTypesAllowed & controller_segment::btt_mask_known) == 0
        || (blockTypesAllowed & ~controller_segment::btt_mask_known) != 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    //TODO: Wert darf nicht auf 0 gesetzt werden, wenn aktuell Aktiv!!
    if ( (deviceAddress == -1 || blockTypesAllowed == controller_segment::btt_none || duration == 0)
        && channelBridge->getBlockTransferState(deviceAddress) == controller_segment::bts_active )
    {
        channelBridge->setLastErrorNumber( deviceAddress, -IV_ERROR_BLOCK_TRANSFER_ACTIVE );
        res = 0;
    }
    else
    {
        channelBridge->setBlockTransferSettings( (deviceAddress == -1) ? 255 : deviceAddress, blockTypesAllowed, duration );
        res = 1;
    }

    channelBridge->logEnd("\t(%i, %i)", res, channelBridge->getLastErrorNumber(deviceAddress));

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCBlockTransferResume( int connectionId, int deviceAddress )
{
    int res = 0;
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return res;
    }

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    res = channelBridge->blockTransferResume( deviceAddress );
    if ( res < IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, -res );
        res = 0;

        if ( channelBridge->getLastErrorNumber( deviceAddress ) != IV_ERROR_BLOCK_TRANSFER_BREAK )
        {
            channelBridge->blockTransferDiscard( deviceAddress );
        }
    }
    else if ( res > IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
    }
    else
    {
        res = 1;
    }

    channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCBlockTransferGetState( int connectionId, int deviceAddress, int *currentType, int *currentState )
{
    int res = 0;
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return res;
    }

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false || currentType == nullptr || currentState == nullptr )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        channelBridge->logEnd( "\t\t\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    if ( channelBridge->getBlockTransferState( deviceAddress ) == controller_segment::bts_inactive )
    {
        *currentType = controller_segment::btt_none;
        *currentState = controller_segment::bts_inactive;
    }
    else
    {
        *currentType = channelBridge->getBlockTransferType( deviceAddress );
        *currentState = channelBridge->getBlockTransferState( deviceAddress );
    }

    res = 1;
    channelBridge->logEnd( "\t%i\t%i\t(%i, %i)", *currentType, *currentState, res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCBlockTransferDiscard( int connectionId, int deviceAddress )
{
    int res = 0;
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return res;
    }

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );
        return res;
    }

    res = channelBridge->blockTransferDiscard( deviceAddress );
    if ( res < IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, -res );
        res = 0;
    }
    else if ( res > IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
    }
    else
    {
        res = 1;
    }

    channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}
