#pragma once
#ifndef DEVICE_STRUCTS_H
#define DEVICE_STRUCTS_H

#pragma pack(push, 1)

/*
	Halten der grundlegenden Dateiinformationen.
	Versionsnummer, Quelldateinamen, Blockgroessen
 */
#define IV_DFFILEDEVICE_SIZE_HEADER                   0x200 // Maximale Groesze fuer Headerdaten
// Fuer das PZE-MasterIV
#define IV_DFFILEDEVICE_VERSION_PZE                   0x00010000
// Fuer das ZK-MasterIV
#define IV_DFFILEDEVICE_VERSION_ZK                    0x00020000
// Fuer das BDE-MasterIV
#define IV_DFFILEDEVICE_VERSION_BDE                   0x00030000
// Fuer das Mobile-MasterIV
#define IV_DFFILEDEVICE_VERSION_MOBIL                 0x00040000
// Fuer das MDE-BoxIV
#define IV_DFFILEDEVICE_VERSION_MDE                   0x00050000
// Fuer das AE-MasterIV
#define IV_DFFILEDEVICE_VERSION_AE                    0x00060000
// Fuer das TimeboyIV
#define IV_DFFILEDEVICE_VERSION_TIMEBOY               0x00070000
// Fuer mehrere Geraetetypen (Information liegt in der Info-Struktur)
#define IV_DFFILEDEVICE_VERSION_MULTIPLE              0x00080000
// Fuer das Exklusiv-Line IV
#define IV_DFFILEDEVICE_VERSION_EXKLUSIV              0x00100000
// Fuer das Flex-MasterIV
#define IV_DFFILEDEVICE_VERSION_FLEX                  0x00200000
// Fuer das Timeboy-Mobil-PZE
#define IV_DFFILEDEVICE_VERSION_TIMEBOY_MOBIL_PZE     0x00400000
// Fuer das Terminal 2.8
#define IV_DFFILEDEVICE_VERSION_TERMINAL_2_8          0x00800000  
// Fuer das Terminal 4.3
#define IV_DFFILEDEVICE_VERSION_TERMINAL_4_3          0x01000000 
// Fuer das IPC-Extension für Vario 5.7
#define IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_VARIO_5_7  0x02000000 
// Fuer das IPC-Extension für Vario/EVO 10,12,15
#define IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_VARIO_10   0x04000000 
// Fuer das IPC-Extension für Q7
#define IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_Q7         0x08000000 
// Fuer das IO-Box
#define IV_DFFILEDEVICE_VERSION_IO_BOX                   0x10000000
// Fuer das ZK-Box
#define IV_DFFILEDEVICE_VERSION_ZK_BOX                   0x20000000
// Fuer das Mobil-Box
#define IV_DFFILEDEVICE_VERSION_MOBIL_BOX                0x40000000
// Die Angabe des Geraetes liegt in der Softwareinfo des Headers.
#define IV_DFFILEDEVICE_VERSION_SOFTWARE_INFO            0x80000000

/*!
    \brief Hält die grundlegenden Informationen über den Inhalt der Gerätedatei.
    
    
*/
struct SDFFileDeviceHeader
{
    unsigned int versionFile;  //!< Dateiversion (Major 1 -> PZE Master IV, Major 2 -> Zutritt Master IV, Major 3 -> BDE Master IV)
    unsigned short sizeData;   //!< Groesse der Headstruktur, gleichzeitig Version: sizeof(SDFFileDeviceHeader), wird auch benoetigt fuer die Berechnung der Position der crcHead - Variablen
    char timestamp[20];        //!< Datetime vom Build, im Format "2005-01-25 07:00:00\0"
    unsigned int crcProgram;   //!< 32bit CRC fuer Programm-Flash
    char versionFirmware[32];  //!< Firmwareversion als String, im Beispielformat: "04.00.23.03\0"
    StructSoftwareInfo softwareInfo; //!< Softwareinformation zur enthaltenen Anwendungsfirmware.
                                     //...
    unsigned short crcHead;    //!< 16bit CRC fuer Head (Immer letzter Wert in der Struktur; 
                               //!< Berechnungsmenge: (sizeData - sizeof(unsigned short)), also ohne sich selbst!)
};

struct SDFFileDeviceHeader1
{
    unsigned int versionFile; //!< Dateiversion (Major 1 -> PZE Master IV, Major 2 -> Zutritt Master IV, Major 3 -> BDE Master IV)
    unsigned short sizeData;  //!< Groesse der Headstruktur: sizeof(SDFFileDeviceHeader), wird benoetigt fuer die Berechnung der Position der crcHead - Variable
    char timestamp[20];       //!< Datetime vom Build, im Format "2005-01-25 07:00:00\0"
    unsigned int crcProgram;  //!< 32bit CRC fuer Programm-Flash
    char versionFirmware[32]; //!< Firmwareversion als String, im Beispielformat: "04.00.23.03\0"
                              //...
    unsigned short crcHead;   //!< 16bit CRC fuer Head (Immer letzter Wert in der Struktur; 
                              //!< Berechnungsmenge: (sizeData - sizeof(unsigned short)), also ohne sich selbst!)
};

#pragma pack(pop)

#endif
